@@ -10,7 +10,7 @@
   See the accompanying LICENSE file.
 */
 
-/** 
+/**
 
 .. module:: apsw
    :synopsis: Python access to SQLite database library
@@ -44,6 +44,7 @@ API Reference
 #define SQLITE_API static
 #define SQLITE_EXTERN static
 #include APSW_USE_SQLITE_AMALGAMATION
+#undef small
 
 /* Fight with SQLite over ndebug */
 #ifdef APSW_NO_NDEBUG
@@ -57,8 +58,8 @@ API Reference
 #include "sqlite3.h"
 #endif
 
-#if SQLITE_VERSION_NUMBER < 3006018
-#error Your SQLite version is too old.  It must be at least 3.6.18
+#if SQLITE_VERSION_NUMBER < 3008011
+#error Your SQLite version is too old.  It must be at least 3.8.11
 #endif
 
 /* system headers */
@@ -113,6 +114,9 @@ static int APSW_Should_Fault(const char *);
 static PyObject *apswmodule;
 
 /* Everything except the module itself is in seperate files */
+#ifdef PYPY_VERSION
+#include "pypycompat.c"
+#endif
 
 /* Augment tracebacks */
 #include "traceback.c"
@@ -168,6 +172,21 @@ getsqliteversion(void)
   return MAKESTR(sqlite3_libversion());
 }
 
+/** .. method:: sqlite3_sourceid() -> string
+
+    Returns the exact checkin information for the SQLite 3 source
+    being used.
+
+    -* sqlite3_sourceid
+*/
+
+static PyObject *
+get_sqlite3_sourceid(void)
+{
+  return MAKESTR(sqlite3_sourceid());
+}
+
+
 /** .. method:: apswversion() -> string
 
   Returns the APSW version.
@@ -183,9 +202,9 @@ getapswversion(void)
   If you use the same :class:`Connection` across threads or use
   multiple :class:`connections <Connection>` accessing the same file,
   then SQLite can `share the cache between them
-  <http://www.sqlite.org/sharedcache.html>`_.  It is :ref:`not
+  <https://sqlite.org/sharedcache.html>`_.  It is :ref:`not
   recommended <sharedcache>` that you use this.
-  
+
   -* sqlite3_enable_shared_cache
 */
 static PyObject *
@@ -241,7 +260,7 @@ static PyObject *
 sqliteshutdown(void)
 {
   int res;
-  
+
   APSW_FAULT_INJECT(ShutdownFail, res=sqlite3_shutdown(), res=SQLITE_NOMEM);
   SET_EXC(res, NULL);
 
@@ -253,15 +272,18 @@ sqliteshutdown(void)
 
 /** .. method:: config(op[, *args])
 
-  :param op: A `configuration operation <http://sqlite.org/c3ref/c_config_chunkalloc.html>`_
+  :param op: A `configuration operation <https://sqlite.org/c3ref/c_config_chunkalloc.html>`_
   :param args: Zero or more arguments as appropriate for *op*
 
-  Many operations don't make sense from a Python program.  Only the
+  Many operations don't make sense from a Python program.  The
   following configuration operations are supported: SQLITE_CONFIG_LOG,
   SQLITE_CONFIG_SINGLETHREAD, SQLITE_CONFIG_MULTITHREAD,
-  SQLITE_CONFIG_SERIALIZED and SQLITE_CONFIG_MEMSTATUS.
+  SQLITE_CONFIG_SERIALIZED, SQLITE_CONFIG_URI, SQLITE_CONFIG_MEMSTATUS,
+  SQLITE_CONFIG_COVERING_INDEX_SCAN, SQLITE_CONFIG_PCACHE_HDRSZ, and
+  SQLITE_CONFIG_PMASZ.
 
-  See :func:`log` for an example of using SQLITE_CONFIG_LOG.
+  See :ref:`tips <diagnostics_tips>` for an example of how to receive
+  log messages (SQLITE_CONFIG_LOG)
 
   -* sqlite3_config
 */
@@ -269,7 +291,7 @@ sqliteshutdown(void)
 #ifdef EXPERIMENTAL
 static PyObject *logger_cb=NULL;
 
-static void 
+static void
 apsw_logger(void *arg, int errcode, const char *message)
 {
   PyGILState_STATE gilstate;
@@ -288,7 +310,7 @@ apsw_logger(void *arg, int errcode, const char *message)
   if(!res)
     {
       AddTraceBackHere(__FILE__, __LINE__, "Call_Logger",
-		       "{s: O, s: i, s: s}", 
+		       "{s: O, s: i, s: s}",
 		       "logger", arg,
 		       "errcode", errcode,
 		       "message", message);
@@ -310,10 +332,8 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
   long opt;
 
   if(PyTuple_GET_SIZE(args)<1 || !PyIntLong_Check(PyTuple_GET_ITEM(args, 0)))
-    {
-      PyErr_Format(PyExc_TypeError, "There should be at least one argument with the first being a number");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "There should be at least one argument with the first being a number");
+
   opt=PyIntLong_AsLong(PyTuple_GET_ITEM(args,0));
   if(PyErr_Occurred())
     return NULL;
@@ -323,13 +343,30 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
     case SQLITE_CONFIG_SINGLETHREAD:
     case SQLITE_CONFIG_MULTITHREAD:
     case SQLITE_CONFIG_SERIALIZED:
+    case SQLITE_CONFIG_URI:
       if(!PyArg_ParseTuple(args, "i", &optdup))
         return NULL;
       assert(opt==optdup);
       res=sqlite3_config( (int)opt );
       break;
-      
+
+    case SQLITE_CONFIG_PCACHE_HDRSZ:
+      {
+        int outval=-1;
+        if(!PyArg_ParseTuple(args, "i", &optdup))
+          return NULL;
+        assert(opt==optdup);
+        res=sqlite3_config( (int)opt, &outval );
+        if(res) {
+          SET_EXC(res, NULL);
+          return NULL;
+        }
+        return PyInt_FromLong(outval);
+      }
+
     case SQLITE_CONFIG_MEMSTATUS:
+    case SQLITE_CONFIG_COVERING_INDEX_SCAN:
+    case SQLITE_CONFIG_PMASZ:
       {
         int boolval;
         if(!PyArg_ParseTuple(args, "ii", &optdup, &boolval))
@@ -338,7 +375,7 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
         res=sqlite3_config( (int)opt, boolval);
         break;
       }
-      
+
     case SQLITE_CONFIG_LOG:
       {
 	PyObject *logger;
@@ -352,8 +389,7 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
 	  }
 	else if(!PyCallable_Check(logger))
 	  {
-	    PyErr_Format(PyExc_TypeError, "Logger should be None or a callable");
-	    return NULL;
+	    return PyErr_Format(PyExc_TypeError, "Logger should be None or a callable");
 	  }
 	else
 	  {
@@ -369,8 +405,7 @@ config(APSW_ARGUNUSED PyObject *self, PyObject *args)
       }
 
     default:
-      PyErr_Format(PyExc_TypeError, "Unknown config type %d", (int)opt);
-      return NULL;
+      return PyErr_Format(PyExc_TypeError, "Unknown config type %d", (int)opt);
     }
 
   SET_EXC(res, NULL);
@@ -421,23 +456,24 @@ memoryhighwater(APSW_ARGUNUSED PyObject *self, PyObject *args)
 }
 
 
-/** .. method:: softheaplimit(bytes)
+/** .. method:: softheaplimit(bytes) -> oldlimit
 
-  Requests SQLite try to keep memory usage below *bytes* bytes.
+  Requests SQLite try to keep memory usage below *bytes* bytes and
+  returns the previous setting.
 
-  -* sqlite3_soft_heap_limit
+  -* sqlite3_soft_heap_limit64
 */
 static PyObject*
 softheaplimit(APSW_ARGUNUSED PyObject *self, PyObject *args)
 {
-  int limit;
+  long long limit, oldlimit;
 
-  if(!PyArg_ParseTuple(args, "i", &limit))
+  if(!PyArg_ParseTuple(args, "L", &limit))
     return NULL;
 
-  sqlite3_soft_heap_limit(limit);
+  oldlimit=sqlite3_soft_heap_limit64(limit);
 
-  Py_RETURN_NONE;
+  return PyLong_FromLongLong(oldlimit);
 }
 
 /** .. method:: randomness(bytes)  -> data
@@ -458,10 +494,7 @@ randomness(APSW_ARGUNUSED PyObject *self, PyObject *args)
   if(!PyArg_ParseTuple(args, "i", &amount))
     return NULL;
   if(amount<0)
-    {
-      PyErr_Format(PyExc_ValueError, "Can't have negative number of bytes");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_ValueError, "Can't have negative number of bytes");
   bytes=PyBytes_FromStringAndSize(NULL, amount);
   if(!bytes) return bytes;
   sqlite3_randomness(amount, PyBytes_AS_STRING(bytes));
@@ -491,32 +524,33 @@ releasememory(APSW_ARGUNUSED PyObject *self, PyObject *args)
 
   Returns current and highwater measurements.
 
-  :param op: A `status parameter <http://sqlite.org/c3ref/c_status_malloc_size.html>`_
+  :param op: A `status parameter <https://sqlite.org/c3ref/c_status_malloc_size.html>`_
   :param reset: If *True* then the highwater is set to the current value
   :returns: A tuple of current value and highwater value
-  
+
   .. seealso::
 
     * :ref:`Status example <example-status>`
 
-  -* sqlite3_status
+  -* sqlite3_status64
 
 */
 static PyObject *
 status(APSW_ARGUNUSED PyObject *self, PyObject *args)
 {
-  int res, op, current=0, highwater=0, reset=0;
+  int res, op, reset=0;
+  sqlite3_int64 current=0, highwater=0;
 
   if(!PyArg_ParseTuple(args, "i|i:status(op, reset=False)", &op, &reset))
     return NULL;
 
-  res=sqlite3_status(op, &current, &highwater, reset);
+  res=sqlite3_status64(op, &current, &highwater, reset);
   SET_EXC(res, NULL);
 
   if(res!=SQLITE_OK)
     return NULL;
 
-  return Py_BuildValue("(ii)", current, highwater);
+  return Py_BuildValue("(LL)", current, highwater);
 }
 
 /** .. method:: vfsnames() -> list(string)
@@ -535,7 +569,7 @@ vfsnames(APSW_ARGUNUSED PyObject *self)
 
   while(vfs)
     {
-      APSW_FAULT_INJECT(vfsnamesfails, 
+      APSW_FAULT_INJECT(vfsnamesfails,
                         str=convertutf8string(vfs->zName),
                         str=PyErr_NoMemory());
       if(!str) goto error;
@@ -555,11 +589,11 @@ vfsnames(APSW_ARGUNUSED PyObject *self)
 
   If you would like to raise an exception that corresponds to a
   particular SQLite `error code
-  <http://sqlite.org/c3ref/c_abort.html>`_ then call this function.
+  <https://sqlite.org/c3ref/c_abort.html>`_ then call this function.
   It also understands `extended error codes
-  <http://sqlite.org/c3ref/c_ioerr_access.html>`_.
+  <https://sqlite.org/c3ref/c_ioerr_access.html>`_.
 
-  For example to raise `SQLITE_IOERR_ACCESS <http://sqlite.org/c3ref/c_ioerr_access.html>`_::
+  For example to raise `SQLITE_IOERR_ACCESS <https://sqlite.org/c3ref/c_ioerr_access.html>`_::
 
     raise apsw.exceptionfor(apsw.SQLITE_IOERR_ACCESS)
 
@@ -571,10 +605,7 @@ getapswexceptionfor(APSW_ARGUNUSED PyObject *self, PyObject *pycode)
   PyObject *result=NULL;
 
   if(!PyIntLong_Check(pycode))
-    {
-      PyErr_Format(PyExc_TypeError, "Argument should be an integer");
-      return NULL;
-    }
+      return PyErr_Format(PyExc_TypeError, "Argument should be an integer");
   code=PyIntLong_AsLong(pycode);
   if(PyErr_Occurred()) return NULL;
 
@@ -586,10 +617,7 @@ getapswexceptionfor(APSW_ARGUNUSED PyObject *self, PyObject *pycode)
         break;
       }
   if(!result)
-    {
-      PyErr_Format(PyExc_ValueError, "%d is not a known error code", code);
-      return result;
-    }
+    return PyErr_Format(PyExc_ValueError, "%d is not a known error code", code);
 
   PyObject_SetAttrString(result, "extendedresult", PyInt_FromLong(code));
   PyObject_SetAttrString(result, "result", PyInt_FromLong(code&0xff));
@@ -649,7 +677,7 @@ apsw_test_reset_rng(APSW_ARGUNUSED PyObject *self)
 #ifdef APSW_TESTFIXTURES
 /* xGetLastError isn't actually called anywhere by SQLite so add a
    manual way of doing so
-   http://www.sqlite.org/cvstrac/tktview?tn=3337 */
+   https://sqlite.org/cvstrac/tktview?tn=3337 */
 
 static PyObject *
 apsw_call_xGetLastError(APSW_ARGUNUSED PyObject *self, PyObject *args)
@@ -690,174 +718,10 @@ apsw_fini(APSW_ARGUNUSED PyObject *self)
 }
 #endif
 
-#ifdef APSW_USE_SQLITE_ASYNCVFS_H
-
-#define SQLITE3ASYNC_API static
-
-#include APSW_USE_SQLITE_ASYNCVFS_H
-
-/** .. method:: async_initialize(parentvfs, makedefault) -> string
-
-  Registers the asyncvfs.  It is safe to call this method multiple
-  times, with subsequent calls having no effect.
-
-  :param parentvfs: Name of the vfs to use for I/O operations.  Use an empty string for the default
-  :param makedefault: Make the async vfs be the default vfs.
-  :returns: A string containg the name of the default vfs.
-
-  Note that this method cannot return string errors as part of
-  exceptions, just generic error codes and messages.  For example if
-  you specify a non-existent parentvfs then you just get
-  :exc:`apsw.SQLError` with the text `error`.  The `source
-  <http://www.sqlite.org/src/finfo?name=ext/async/sqlite3async.h>`__
-  lists what error codes could be returned.
-
-  .. seealso::
-
-  * :ref:`ext-asyncvfs`
-*/
-static PyObject *
-apsw_async_initialize(APSW_ARGUNUSED PyObject *self, PyObject *args)
-{
-  char *vfsname=NULL;
-  int makedefault=0;
-  int res;
-
-  if(!PyArg_ParseTuple(args, "esi", STRENCODING, &vfsname, &makedefault))
-    return NULL;
-
-  if(!*vfsname)
-    {
-      PyMem_Free(vfsname);
-      vfsname=NULL;
-    }
-
-  _PYSQLITE_CALL_V(res=sqlite3async_initialize(vfsname, makedefault));
-  PyMem_Free(vfsname);
-  if(res!=SQLITE_OK)
-    {
-      SET_EXC(res, NULL);
-      return NULL;
-    }
-
-  return convertutf8string(SQLITEASYNC_VFSNAME);
-}
-
-/** .. method:: async_shutdown()
-
-  Unregisters the asyncvfs.  It is safe to call this method multiple
-  times with subsequent calls having no effect.  It is not recommended
-  that you call this method.  It deallocates everything immediately
-  making no checks if anything is in use.  If you want to call it then
-  only do so once you are absolutely sure that all databases, files,
-  vfs, threads and other objects it references have been closed and
-  freed.
-*/
-static PyObject *
-apsw_async_shutdown(APSW_ARGUNUSED PyObject *self)
-{
-  _PYSQLITE_CALL_V(sqlite3async_shutdown());
-
-  Py_RETURN_NONE;
-}
-
-/** .. method:: async_control(op[, params])
-
-    :param op: The operation code (eg SQLITE_ASYNC_HALT)
-    :param params: Optional additional parameters depending on the opcode.
-    :returns: None or additional information requested
-
-    This table summarises the parameters taken and value returned
-    depending on the operation.
-
-    +-----------------------------------+--------------+---------+
-    | op                                | params       | result  |
-    +===================================+==============+=========+
-    | SQLITEASYNC_HALT                  | int          | None    |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_GET_HALT              |              | int     |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_DELAY                 | int          | None    |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_GET_DELAY             |              | int     |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_LOCKFILES             | int          | None    |
-    +-----------------------------------+--------------+---------+
-    | SQLITEASYNC_GET_LOCKFILES         |              | int     |
-    +-----------------------------------+--------------+---------+
-*/
-static PyObject *
-apsw_async_control(APSW_ARGUNUSED PyObject *self, PyObject *args)
-{
-  int op, inparam, outparam,res;
-
-  if(!PyTuple_Check(args) || PyTuple_GET_SIZE(args)<1)
-    {
-      PyErr_Format(PyExc_TypeError, "Args should be a tuple of at least one item");
-      return NULL;
-    }
-  if(!PyIntLong_Check(PyTuple_GET_ITEM(args, 0)))
-    {
-      PyErr_Format(PyExc_TypeError, "Arg 0 must be a number");
-      return NULL;
-    }
-
-  switch(PyIntLong_AsLong(PyTuple_GET_ITEM(args, 0)))
-    {
-    case SQLITEASYNC_HALT:
-    case SQLITEASYNC_DELAY:
-    case SQLITEASYNC_LOCKFILES:
-      if(!PyArg_ParseTuple(args, "ii", &op, &inparam))
-	return NULL;
-      _PYSQLITE_CALL_V(res=sqlite3async_control(op, inparam));
-      if(res!=SQLITE_OK)
-	{
-	  SET_EXC(res, NULL);
-	  return NULL;
-	}
-      Py_RETURN_NONE;
-
-    case SQLITEASYNC_GET_HALT:
-    case SQLITEASYNC_GET_DELAY:
-    case SQLITEASYNC_GET_LOCKFILES:
-      op=PyIntLong_AsLong(PyTuple_GET_ITEM(args, 0));
-      assert(!PyErr_Occurred());
-      _PYSQLITE_CALL_V(res=sqlite3async_control(op, &outparam));
-      APSW_FAULT_INJECT(AsyncControlFails,,res=SQLITE_NOMEM);
-      if(res!=SQLITE_OK)
-	{
-	  SET_EXC(res, NULL);
-	  return NULL;
-	}
-      return PyInt_FromLong(outparam);
-
-    default:
-      PyErr_Format(PyExc_ValueError, "Unknown operation argument");
-      return NULL;
-    }
-}
-
-/** .. method:: async_run()
-
-  Call this method from a worker thread and it will do all the
-  asynchronous I/O.  You can get the function to exit by calling
-  :func:`async_control` with a first parameter of :const:`SQLITEASYNC_HALT` and
-  a second parameter of :const:`SQLITEASYNC_HALT_NOW`.
-*/
-static PyObject *
-apsw_async_run(APSW_ARGUNUSED PyObject *self)
-{
-  _PYSQLITE_CALL_V(sqlite3async_run());
-
-  Py_RETURN_NONE;
-}
-
-
-#endif
 
 #ifdef APSW_FORK_CHECKER
 
-/* 
+/*
    We want to verify that SQLite objects are not used across forks.
    One way is to modify all calls to SQLite to do the checking but
    this is a pain as well as a performance hit.  Instead we use the
@@ -866,7 +730,7 @@ apsw_async_run(APSW_ARGUNUSED PyObject *self)
 
    Our diverted functions check the process id on calls and set the
    process id on allocating a mutex.  We have to avoid the checks for
-   the static mutexes.  
+   the static mutexes.
 
    This code also doesn't bother with some things like checking malloc
    results.  It is intended to only be used to verify correctness with
@@ -875,7 +739,7 @@ apsw_async_run(APSW_ARGUNUSED PyObject *self)
    good thing - you will really be sure there is a problem!
  */
 
-typedef struct 
+typedef struct
 {
   pid_t pid;
   sqlite3_mutex *underlying_mutex;
@@ -920,7 +784,7 @@ apsw_xMutexAlloc(int which)
       {
 	apsw_mutex *am;
 	sqlite3_mutex *m=apsw_orig_mutex_methods.xMutexAlloc(which);
-	
+
 	if(!m) return m;
 
 	am=malloc(sizeof(apsw_mutex));
@@ -1034,7 +898,7 @@ static sqlite3_mutex_methods apsw_mutex_methods=
 
   SQLite does not allow the use of database connections across `forked
   <http://en.wikipedia.org/wiki/Fork_(operating_system)>`__ processes
-  (see the `SQLite FAQ Q6 <http://www.sqlite.org/faq.html#q6>`__).
+  (see the `SQLite FAQ Q6 <https://sqlite.org/faq.html#q6>`__).
   (Forking creates a child process that is a duplicate of the parent
   including the state of all data structures in the program.  If you
   do this to SQLite then parent and child would both consider
@@ -1088,10 +952,10 @@ apsw_fork_checker(APSW_ARGUNUSED PyObject *self)
   /* then do a shutdown as we can't get or change mutex while sqlite is running */
   rc=sqlite3_shutdown();
   if(rc) goto fail;
-  
+
   rc=sqlite3_config(SQLITE_CONFIG_GETMUTEX, &apsw_orig_mutex_methods);
   if(rc) goto fail;
-  
+
   rc=sqlite3_config(SQLITE_CONFIG_MUTEX, &apsw_mutex_methods);
   if(rc) goto fail;
 
@@ -1101,7 +965,7 @@ apsw_fork_checker(APSW_ARGUNUSED PyObject *self)
 
  ok:
   Py_RETURN_NONE;
-  
+
  fail:
   assert(rc!=SQLITE_OK);
   SET_EXC(rc, NULL);
@@ -1152,13 +1016,134 @@ get_compile_options(void)
 }
 
 
+/** .. method:: format_sql_value(value) -> string
+
+  Returns a Python string (unicode) representing the supplied value in
+  SQL syntax.  Python 2 note: You must supply unicode strings not
+  plain strings.
+
+*/
+static PyObject*
+formatsqlvalue(APSW_ARGUNUSED PyObject *self, PyObject *value)
+{
+  /* NULL/None */
+  if(value==Py_None)
+    {
+      static PyObject *nullstr;
+      if(!nullstr) nullstr=PyObject_Unicode(MAKESTR("NULL"));
+      Py_INCREF(nullstr);
+      return nullstr;
+    }
+  /* Integer/Long/Float */
+  if(PyIntLong_Check(value)  /* ::TODO:: verify L is not appended in py 2.3 and similar vintage */
+     || PyFloat_Check(value))
+    return PyObject_Unicode(value);
+#if PY_MAJOR_VERSION<3
+  /* We don't support plain strings only unicode */
+  if(PyString_Check(value))
+    return PyErr_Format(PyExc_TypeError, "Old plain strings not supported - use unicode");
+#endif
+  /* Unicode */
+  if(PyUnicode_Check(value))
+    {
+      /* We optimize for the default case of there being no nuls or single quotes */
+      PyObject *unires;
+      Py_UNICODE *res;
+      Py_ssize_t left;
+      unires=PyUnicode_FromUnicode(NULL, PyUnicode_GET_SIZE(value)+2);
+      if(!unires) return NULL;
+      APSW_UNICODE_READY(unires, return NULL);
+      res=PyUnicode_AS_UNICODE(unires);
+      *res++='\'';
+      memcpy(res, PyUnicode_AS_UNICODE(value), PyUnicode_GET_DATA_SIZE(value));
+      res+=PyUnicode_GET_SIZE(value);
+      *res++='\'';
+      /* Now look for nuls and single quotes */
+      res=PyUnicode_AS_UNICODE(unires)+1;
+      left=PyUnicode_GET_SIZE(value);
+      for(;left;left--,res++)
+	{
+	  if(*res=='\'' || *res==0)
+	    {
+	      /* we add one char for ' and 10 for null */
+	      const int moveamount=*res=='\''?1:10;
+	      int retval;
+	      APSW_FAULT_INJECT(FormatSQLValueResizeFails,
+				retval=PyUnicode_Resize(&unires, PyUnicode_GET_SIZE(unires)+moveamount),
+				retval=PyUnicode_Resize(&unires, -17)
+				);
+	      if(retval==-1)
+		{
+		  Py_DECREF(unires);
+		  return NULL;
+		}
+	      res=PyUnicode_AS_UNICODE(unires)+(PyUnicode_GET_SIZE(unires)-left-moveamount-1);
+	      memmove(res+moveamount, res, sizeof(Py_UNICODE)*(left+1));
+	      if(*res==0)
+		{
+		  *res++='\'';
+		  *res++='|'; *res++='|';
+		  *res++='X'; *res++='\''; *res++='0'; *res++='0'; *res++='\'';
+		  *res++='|'; *res++='|';
+		  *res='\'';
+		}
+	      else
+		res++;
+	    }
+	}
+      return unires;
+    }
+  /* Blob */
+  if(
+#if PY_MAJOR_VERSION<3
+     PyBuffer_Check(value)
+#else
+     PyBytes_Check(value)
+#endif
+     )
+    {
+      const unsigned char *buffer;
+      Py_ssize_t buflen;
+      int asrb;
+      PyObject *unires;
+      Py_UNICODE *res;
+#define _HEXDIGITS
+
+      asrb=PyObject_AsReadBuffer(value, (const void**)&buffer, &buflen);
+      APSW_FAULT_INJECT(FormatSQLValueAsReadBufferFails,
+			,
+			(PyErr_NoMemory(), asrb=-1));
+      if(asrb!=0)
+	return NULL;
+      /* 3 is X, ', '  */
+      APSW_FAULT_INJECT(FormatSQLValuePyUnicodeFromUnicodeFails,
+			unires=PyUnicode_FromUnicode(NULL, buflen*2+3),
+			unires=PyErr_NoMemory());
+      if(!unires)
+	return NULL;
+      APSW_UNICODE_READY(unires, return NULL);
+      res=PyUnicode_AS_UNICODE(unires);
+      *res++='X';
+      *res++='\'';
+      /* About the billionth time I have written a hex conversion routine */
+      for(;buflen;buflen--)
+	{
+	  *res++="0123456789ABCDEF"[(*buffer)>>4];
+	  *res++="0123456789ABCDEF"[(*buffer++)&0x0f];
+	}
+      *res++='\'';
+      return unires;
+    }
+
+  return PyErr_Format(PyExc_TypeError, "Unsupported type");
+}
+
 /** .. automethod:: main()
 
   Sphinx automethod is too stupid, so this text is replaced by
   my code with the actual docstring from tools.py:main().
 */
 
-#ifdef EXPERIMENTAL
 /** .. method:: log(level, message)
 
     Calls the SQLite logging interface.  Note that you must format the
@@ -1166,18 +1151,8 @@ get_compile_options(void)
 
         apsw.log(apsw.SQLITE_NOMEM, "Need %d bytes of memory" % (1234,))
 
-    To set your own logger use::
-
-        def handler(errcode, message):
-           print errcode, message
-        apsw.config(apsw.SQLITE_CONFIG_LOG, handler)
-
-    *handler* will be called with two arguments.  The first will be a
-    numeric error code and the second will be a message string.  Note
-    that the handler has to be set before any other calls to SQLite.
-    Once SQLite is initialised you cannot change the logger - a
-    :exc:`MisuseError` will happen (this restriction is in SQLite not
-    APSW).
+    See :ref:`tips <diagnostics_tips>` for an example of how to
+    receive log messages.
 
     -* sqlite3_log
  */
@@ -1192,10 +1167,10 @@ apsw_log(APSW_ARGUNUSED PyObject *self, PyObject *args)
   PyMem_Free(message);
   Py_RETURN_NONE;
 }
-#endif
-
 
 static PyMethodDef module_methods[] = {
+  {"sqlite3_sourceid", (PyCFunction)get_sqlite3_sourceid, METH_NOARGS,
+   "Return the source identification of the SQLite library"},
   {"sqlitelibversion", (PyCFunction)getsqliteversion, METH_NOARGS,
    "Return the version of the SQLite library"},
   {"apswversion", (PyCFunction)getapswversion, METH_NOARGS,
@@ -1208,6 +1183,8 @@ static PyMethodDef module_methods[] = {
    "Initialize SQLite library"},
   {"shutdown", (PyCFunction)sqliteshutdown, METH_NOARGS,
    "Shutdown SQLite library"},
+  {"format_sql_value", (PyCFunction)formatsqlvalue, METH_O,
+   "Formats a SQL value as a string"},
 #ifdef EXPERIMENTAL
   {"config", (PyCFunction)config, METH_VARARGS,
    "Calls sqlite3_config"},
@@ -1240,16 +1217,6 @@ static PyMethodDef module_methods[] = {
   {"_fini", (PyCFunction)apsw_fini, METH_NOARGS,
    "Frees all caches and recycle lists"},
 #endif
-#ifdef APSW_USE_SQLITE_ASYNCVFS_H
-  {"async_initialize", (PyCFunction)apsw_async_initialize, METH_VARARGS,
-   "Initializes the asyncvfs extension"},
-  {"async_shutdown", (PyCFunction)apsw_async_shutdown, METH_NOARGS,
-   "Frees all asyncvfs objects"},
-  {"async_control", (PyCFunction)apsw_async_control, METH_VARARGS,
-   "Control operation of asyncvfs"},
-  {"async_run", (PyCFunction)apsw_async_run, METH_NOARGS,
-   "Does the background async I/O"},
-#endif
 #ifdef APSW_FORK_CHECKER
   {"fork_checker", (PyCFunction)apsw_fork_checker, METH_NOARGS,
    "Installs fork checking code"},
@@ -1257,14 +1224,13 @@ static PyMethodDef module_methods[] = {
   {0, 0, 0, 0}  /* Sentinel */
 };
 
-
 static void add_shell(PyObject *module);
 
 
 #if PY_MAJOR_VERSION >= 3
 static struct PyModuleDef apswmoduledef={
   PyModuleDef_HEAD_INIT,
-  "apsw", 
+  "apsw",
   NULL,
   -1,
   module_methods,
@@ -1278,7 +1244,7 @@ static struct PyModuleDef apswmoduledef={
 
 PyMODINIT_FUNC
 #if PY_MAJOR_VERSION < 3
-initapsw(void) 
+initapsw(void)
 #else
 PyInit_apsw(void)
 #endif
@@ -1305,6 +1271,7 @@ PyInit_apsw(void)
         || PyType_Ready(&APSWBlobType) <0
         || PyType_Ready(&APSWVFSType) <0
         || PyType_Ready(&APSWVFSFileType) <0
+	|| PyType_Ready(&APSWURIFilenameType) <0
         || PyType_Ready(&APSWStatementType) <0
         || PyType_Ready(&APSWBufferType) <0
         || PyType_Ready(&FunctionCBInfoType) <0
@@ -1326,13 +1293,15 @@ PyInit_apsw(void)
 
     if (m == NULL)  goto fail;
 
+    Py_INCREF(m);
+
     if(init_exceptions(m)) goto fail;
 
     Py_INCREF(&ConnectionType);
     PyModule_AddObject(m, "Connection", (PyObject *)&ConnectionType);
-    
+
     /* we don't add cursor, blob or backup to the module since users shouldn't be able to instantiate them directly */
-    
+
     Py_INCREF(&ZeroBlobBindType);
     PyModule_AddObject(m, "zeroblob", (PyObject *)&ZeroBlobBindType);
 
@@ -1340,7 +1309,10 @@ PyInit_apsw(void)
     PyModule_AddObject(m, "VFS", (PyObject*)&APSWVFSType);
     Py_INCREF(&APSWVFSFileType);
     PyModule_AddObject(m, "VFSFile", (PyObject*)&APSWVFSFileType);
-    
+    Py_INCREF(&APSWURIFilenameType);
+    PyModule_AddObject(m, "URIFilename", (PyObject*)&APSWURIFilenameType);
+
+
     /** .. attribute:: connection_hooks
 
        The purpose of the hooks is to allow the easy registration of
@@ -1371,16 +1343,16 @@ PyInit_apsw(void)
 
     */
     PyModule_AddIntConstant(m, "SQLITE_VERSION_NUMBER", SQLITE_VERSION_NUMBER);
-    
+
 
     /** .. attribute:: using_amalgamation
 
     If True then `SQLite amalgamation
-    <http://www.sqlite.org/cvstrac/wiki?p=TheAmalgamation>`__ is in
+    <https://sqlite.org/cvstrac/wiki?p=TheAmalgamation>`__ is in
     use (statically compiled into APSW).  Using the amalgamation means
     that SQLite shared libraries are not used and will not affect your
     code.
-    
+
     */
 
 #ifdef APSW_USE_SQLITE_AMALGAMATION
@@ -1391,7 +1363,7 @@ PyInit_apsw(void)
     PyModule_AddObject(m, "using_amalgamation", Py_False);
 #endif
 
-    /** 
+    /**
 
 .. _sqliteconstants:
 
@@ -1399,7 +1371,7 @@ SQLite constants
 ================
 
 SQLite has `many constants
-<http://sqlite.org/c3ref/constlist.html>`_ used in various
+<https://sqlite.org/c3ref/constlist.html>`_ used in various
 interfaces.  To use a constant such as :const:`SQLITE_OK`, just
 use ``apsw.SQLITE_OK``.
 
@@ -1411,8 +1383,8 @@ numeric value, or supply a numeric value and get the corresponding
 string. These can help improve diagnostics/logging, calling other
 modules etc. For example::
 
-      apsw.mapping_authorizer_function["SQLITE_READ"]=20
-      apsw.mapping_authorizer_function[20]="SQLITE_READ"
+      apsw.mapping_authorizer_function["SQLITE_READ"] == 20
+      apsw.mapping_authorizer_function[20] == "SQLITE_READ"
 
 
     */
@@ -1434,7 +1406,7 @@ modules etc. For example::
       ADDINT(SQLITE_IGNORE),
       ADDINT(SQLITE_OK),
       END,
-      
+
       DICT("mapping_authorizer_function"),
       ADDINT(SQLITE_CREATE_INDEX),
       ADDINT(SQLITE_CREATE_TABLE),
@@ -1469,6 +1441,7 @@ modules etc. For example::
       ADDINT(SQLITE_DROP_VTABLE),
       ADDINT(SQLITE_FUNCTION),
       ADDINT(SQLITE_SAVEPOINT),
+      ADDINT(SQLITE_RECURSIVE),
       END,
 
       /* vtable best index constraints */
@@ -1501,6 +1474,41 @@ modules etc. For example::
       ADDINT(SQLITE_IOERR_CLOSE),
       ADDINT(SQLITE_IOERR_DIR_CLOSE),
       ADDINT(SQLITE_LOCKED_SHAREDCACHE),
+      ADDINT(SQLITE_BUSY_RECOVERY),
+      ADDINT(SQLITE_CANTOPEN_NOTEMPDIR),
+      ADDINT(SQLITE_IOERR_SHMOPEN),
+      ADDINT(SQLITE_IOERR_SHMSIZE),
+      ADDINT(SQLITE_IOERR_SHMLOCK),
+      ADDINT(SQLITE_CORRUPT_VTAB),
+      ADDINT(SQLITE_IOERR_SEEK),
+      ADDINT(SQLITE_IOERR_SHMMAP),
+      ADDINT(SQLITE_READONLY_CANTLOCK),
+      ADDINT(SQLITE_READONLY_RECOVERY),
+      ADDINT(SQLITE_ABORT_ROLLBACK),
+      ADDINT(SQLITE_CANTOPEN_ISDIR),
+      ADDINT(SQLITE_CANTOPEN_FULLPATH),
+      ADDINT(SQLITE_IOERR_DELETE_NOENT),
+      ADDINT(SQLITE_CONSTRAINT_CHECK),
+      ADDINT(SQLITE_CONSTRAINT_COMMITHOOK),
+      ADDINT(SQLITE_CONSTRAINT_FOREIGNKEY),
+      ADDINT(SQLITE_CONSTRAINT_FUNCTION),
+      ADDINT(SQLITE_CONSTRAINT_NOTNULL),
+      ADDINT(SQLITE_CONSTRAINT_PRIMARYKEY),
+      ADDINT(SQLITE_CONSTRAINT_TRIGGER),
+      ADDINT(SQLITE_CONSTRAINT_UNIQUE),
+      ADDINT(SQLITE_CONSTRAINT_VTAB),
+      ADDINT(SQLITE_READONLY_ROLLBACK),
+      ADDINT(SQLITE_IOERR_MMAP),
+      ADDINT(SQLITE_NOTICE_RECOVER_ROLLBACK),
+      ADDINT(SQLITE_NOTICE_RECOVER_WAL),
+      ADDINT(SQLITE_BUSY_SNAPSHOT),
+      ADDINT(SQLITE_IOERR_GETTEMPPATH),
+      ADDINT(SQLITE_WARNING_AUTOINDEX),
+      ADDINT(SQLITE_CANTOPEN_CONVPATH),
+      ADDINT(SQLITE_IOERR_CONVPATH),
+      ADDINT(SQLITE_CONSTRAINT_ROWID),
+      ADDINT(SQLITE_READONLY_DBMOVED),
+      ADDINT(SQLITE_AUTH_USER),
       END,
 
       /* error codes */
@@ -1532,6 +1540,11 @@ modules etc. For example::
       ADDINT(SQLITE_NOTADB),
       ADDINT(SQLITE_NOTFOUND),
       ADDINT(SQLITE_TOOBIG),
+      ADDINT(SQLITE_NOTICE),
+      ADDINT(SQLITE_WARNING),
+      /* you can't get these from apsw code but present for completeness */
+      ADDINT(SQLITE_DONE),
+      ADDINT(SQLITE_ROW),
       END,
 
       /* open flags */
@@ -1553,6 +1566,9 @@ modules etc. For example::
       ADDINT(SQLITE_OPEN_PRIVATECACHE),
       ADDINT(SQLITE_OPEN_SHAREDCACHE),
       ADDINT(SQLITE_OPEN_AUTOPROXY),
+      ADDINT(SQLITE_OPEN_WAL),
+      ADDINT(SQLITE_OPEN_URI),
+      ADDINT(SQLITE_OPEN_MEMORY),
       END,
 
       /* limits */
@@ -1568,7 +1584,8 @@ modules etc. For example::
       ADDINT(SQLITE_LIMIT_LIKE_PATTERN_LENGTH),
       ADDINT(SQLITE_LIMIT_VARIABLE_NUMBER),
       ADDINT(SQLITE_LIMIT_TRIGGER_DEPTH),
-      /* We don't include the MAX limits - see http://code.google.com/p/apsw/issues/detail?id=17 */
+      ADDINT(SQLITE_LIMIT_WORKER_THREADS),
+      /* We don't include the MAX limits - see https://github.com/rogerbinns/apsw/issues/17 */
       END,
 
       DICT("mapping_config"),
@@ -1585,16 +1602,23 @@ modules etc. For example::
       ADDINT(SQLITE_CONFIG_GETMUTEX),
       ADDINT(SQLITE_CONFIG_LOOKASIDE),
       ADDINT(SQLITE_CONFIG_LOG),
-#ifdef SQLITE_CONFIG_GETPCACHE
       ADDINT(SQLITE_CONFIG_GETPCACHE),
-#endif
-#ifdef SQLITE_CONFIG_PCACHE
       ADDINT(SQLITE_CONFIG_PCACHE),
-#endif
+      ADDINT(SQLITE_CONFIG_URI),
+      ADDINT(SQLITE_CONFIG_PCACHE2),
+      ADDINT(SQLITE_CONFIG_GETPCACHE2),
+      ADDINT(SQLITE_CONFIG_COVERING_INDEX_SCAN),
+      ADDINT(SQLITE_CONFIG_SQLLOG),
+      ADDINT(SQLITE_CONFIG_MMAP_SIZE),
+      ADDINT(SQLITE_CONFIG_WIN32_HEAPSIZE),
+      ADDINT(SQLITE_CONFIG_PCACHE_HDRSZ),
+      ADDINT(SQLITE_CONFIG_PMASZ),
       END,
 
       DICT("mapping_db_config"),
       ADDINT(SQLITE_DBCONFIG_LOOKASIDE),
+      ADDINT(SQLITE_DBCONFIG_ENABLE_FKEY),
+      ADDINT(SQLITE_DBCONFIG_ENABLE_TRIGGER),
       END,
 
       DICT("mapping_status"),
@@ -1607,10 +1631,22 @@ modules etc. For example::
       ADDINT(SQLITE_STATUS_PARSER_STACK),
       ADDINT(SQLITE_STATUS_PAGECACHE_SIZE),
       ADDINT(SQLITE_STATUS_SCRATCH_SIZE),
+      ADDINT(SQLITE_STATUS_MALLOC_COUNT),
       END,
 
       DICT("mapping_db_status"),
       ADDINT(SQLITE_DBSTATUS_LOOKASIDE_USED),
+      ADDINT(SQLITE_DBSTATUS_CACHE_USED),
+      ADDINT(SQLITE_DBSTATUS_MAX),
+      ADDINT(SQLITE_DBSTATUS_SCHEMA_USED),
+      ADDINT(SQLITE_DBSTATUS_STMT_USED),
+      ADDINT(SQLITE_DBSTATUS_LOOKASIDE_HIT),
+      ADDINT(SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL),
+      ADDINT(SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE),
+      ADDINT(SQLITE_DBSTATUS_CACHE_HIT),
+      ADDINT(SQLITE_DBSTATUS_CACHE_MISS),
+      ADDINT(SQLITE_DBSTATUS_CACHE_WRITE),
+      ADDINT(SQLITE_DBSTATUS_DEFERRED_FKS),
       END,
 
       DICT("mapping_locking_level"),
@@ -1639,6 +1675,9 @@ modules etc. For example::
       ADDINT(SQLITE_IOCAP_ATOMIC64K),
       ADDINT(SQLITE_IOCAP_SAFE_APPEND),
       ADDINT(SQLITE_IOCAP_SEQUENTIAL),
+      ADDINT(SQLITE_IOCAP_UNDELETABLE_WHEN_OPEN),
+      ADDINT(SQLITE_IOCAP_POWERSAFE_OVERWRITE),
+      ADDINT(SQLITE_IOCAP_IMMUTABLE),
       END,
 
       DICT("mapping_sync"),
@@ -1647,33 +1686,63 @@ modules etc. For example::
       ADDINT(SQLITE_SYNC_DATAONLY),
       END,
 
-      DICT("mapping_file_control"),
-      ADDINT(SQLITE_FCNTL_LOCKSTATE),
-      ADDINT(SQLITE_GET_LOCKPROXYFILE),
-      ADDINT(SQLITE_SET_LOCKPROXYFILE),
-      ADDINT(SQLITE_LAST_ERRNO),
-      END
-
-#ifdef APSW_USE_SQLITE_ASYNCVFS_H
-      ,
-      DICT("mapping_asyncvfs_control"),
-      ADDINT(SQLITEASYNC_HALT),
-      ADDINT(SQLITEASYNC_GET_HALT),
-      ADDINT(SQLITEASYNC_DELAY),
-      ADDINT(SQLITEASYNC_GET_DELAY),
-      ADDINT(SQLITEASYNC_LOCKFILES),
-      ADDINT(SQLITEASYNC_GET_LOCKFILES),
+      DICT("mapping_wal_checkpoint"),
+      ADDINT(SQLITE_CHECKPOINT_PASSIVE),
+      ADDINT(SQLITE_CHECKPOINT_FULL),
+      ADDINT(SQLITE_CHECKPOINT_RESTART),
+      ADDINT(SQLITE_CHECKPOINT_TRUNCATE),
       END,
 
-      DICT("mapping_asyncvfs_control_halt"),
-      ADDINT(SQLITEASYNC_HALT_NEVER),
-      ADDINT(SQLITEASYNC_HALT_NOW),
-      ADDINT(SQLITEASYNC_HALT_IDLE),
+      DICT("mapping_file_control"),
+      ADDINT(SQLITE_FCNTL_LOCKSTATE),
+      ADDINT(SQLITE_FCNTL_SIZE_HINT),
+      ADDINT(SQLITE_FCNTL_CHUNK_SIZE),
+      ADDINT(SQLITE_FCNTL_FILE_POINTER),
+      ADDINT(SQLITE_FCNTL_SYNC_OMITTED),
+      ADDINT(SQLITE_FCNTL_PERSIST_WAL),
+      ADDINT(SQLITE_FCNTL_WIN32_AV_RETRY),
+      ADDINT(SQLITE_FCNTL_OVERWRITE),
+      ADDINT(SQLITE_FCNTL_POWERSAFE_OVERWRITE),
+      ADDINT(SQLITE_FCNTL_VFSNAME),
+      ADDINT(SQLITE_FCNTL_PRAGMA),
+      ADDINT(SQLITE_FCNTL_BUSYHANDLER),
+      ADDINT(SQLITE_FCNTL_TEMPFILENAME),
+      ADDINT(SQLITE_FCNTL_MMAP_SIZE),
+      ADDINT(SQLITE_FCNTL_TRACE),
+      ADDINT(SQLITE_FCNTL_COMMIT_PHASETWO),
+      ADDINT(SQLITE_FCNTL_HAS_MOVED),
+      ADDINT(SQLITE_FCNTL_SYNC),
+      ADDINT(SQLITE_FCNTL_WIN32_SET_HANDLE),
+      ADDINT(SQLITE_FCNTL_LAST_ERRNO),
+      ADDINT(SQLITE_FCNTL_WAL_BLOCK),
+      ADDINT(SQLITE_FCNTL_GET_LOCKPROXYFILE),
+      ADDINT(SQLITE_FCNTL_SET_LOCKPROXYFILE),
+      ADDINT(SQLITE_FCNTL_RBU),
+      ADDINT(SQLITE_FCNTL_ZIPVFS),
+      END,
+
+      DICT("mapping_conflict_resolution_modes"),
+      ADDINT(SQLITE_ROLLBACK),
+      ADDINT(SQLITE_IGNORE),
+      ADDINT(SQLITE_FAIL),
+      ADDINT(SQLITE_ABORT),
+      ADDINT(SQLITE_REPLACE),
+      END,
+
+      DICT("mapping_virtual_table_configuration_options"),
+      ADDINT(SQLITE_VTAB_CONSTRAINT_SUPPORT),
+      END,
+
+      DICT("mapping_xshmlock_flags"),
+      ADDINT(SQLITE_SHM_EXCLUSIVE),
+      ADDINT(SQLITE_SHM_LOCK),
+      ADDINT(SQLITE_SHM_SHARED),
+      ADDINT(SQLITE_SHM_UNLOCK),
       END
-#endif
+
       };
- 
- 
+
+
  for(i=0;i<sizeof(integers)/sizeof(integers[0]); i++)
    {
      const char *name=integers[i].name;
@@ -1729,7 +1798,7 @@ modules etc. For example::
 
  fail:
     Py_XDECREF(m);
-    return 
+    return
 #if PY_MAJOR_VERSION >= 3
           NULL
 #endif
@@ -1739,6 +1808,7 @@ modules etc. For example::
 static void
 add_shell(PyObject *apswmodule)
 {
+#ifndef PYPY_VERSION
   PyObject *res=NULL, *maindict=NULL, *apswdict, *msvciscrap=NULL;
 
   maindict=PyModule_GetDict(PyImport_AddModule("__main__"));
@@ -1761,6 +1831,7 @@ add_shell(PyObject *apswmodule)
   assert(res);
   Py_XDECREF(res);
   Py_XDECREF(msvciscrap);
+#endif
 }
 
 #ifdef APSW_TESTFIXTURES
@@ -1777,9 +1848,9 @@ APSW_Should_Fault(const char *name)
     PyObject_SetAttrString(apswmodule, "faultdict", PyDict_New());
 
   value=MAKESTR(name);
-  
+
   faultdict=PyObject_GetAttrString(apswmodule, "faultdict");
-  
+
   truthval=PyDict_GetItem(faultdict, value);
   if(!truthval)
     goto finally;
@@ -1796,9 +1867,3 @@ APSW_Should_Fault(const char *name)
   return res;
 }
 #endif
-
-/* async vfs */
-#ifdef APSW_USE_SQLITE_ASYNCVFS_C
-#define SQLITE_ENABLE_ASYNCIO
-#include APSW_USE_SQLITE_ASYNCVFS_C
-#endif
