@@ -5,26 +5,26 @@
 */
 
 
-/** 
+/**
 .. _blobio:
 
 Blob Input/Output
 *****************
 
 A `blob <http://en.wikipedia.org/wiki/Binary_large_object>`_ is a
-SQLite `datatype <http://www.sqlite.org/datatype3.html>`_ representing
+SQLite `datatype <https://sqlite.org/datatype3.html>`_ representing
 a sequence of bytes.  It can be zero or more bytes in size.
 
 SQLite blobs have an absolute maximum size of 2GB and a `default
-maximum size <http://www.sqlite.org/c3ref/c_limit_attached.html>`_ of
+maximum size <https://sqlite.org/c3ref/c_limit_attached.html>`_ of
 1GB.
 
 An alternate approach to using blobs is to store the data in files and
 store the filename in the database.  Doing so loses the `ACID
-<http://www.sqlite.org/transactional.html>`_ properties of SQLite.
+<https://sqlite.org/transactional.html>`_ properties of SQLite.
 
 */
-   
+
 
 /* ZEROBLOB CODE */
 
@@ -77,7 +77,7 @@ ZeroBlobBind_init(ZeroBlobBind *self, PyObject *args, PyObject *kwargs)
       PyErr_Format(PyExc_TypeError, "Zeroblob constructor does not take keyword arguments");
       return -1;
     }
-  
+
   if(!PyArg_ParseTuple(args, "i", &n))
     return -1;
 
@@ -92,7 +92,7 @@ ZeroBlobBind_init(ZeroBlobBind *self, PyObject *args, PyObject *kwargs)
 }
 
 /** .. method:: length() -> int
- 
+
   Size of zero blob in bytes.
 */
 static PyObject *
@@ -112,7 +112,7 @@ static PyTypeObject ZeroBlobBindType = {
     "apsw.zeroblob",           /*tp_name*/
     sizeof(ZeroBlobBind),      /*tp_basicsize*/
     0,                         /*tp_itemsize*/
-    0,                         /*tp_dealloc*/ 
+    0,                         /*tp_dealloc*/
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
@@ -180,14 +180,14 @@ static PyTypeObject APSWBlobType;
   This object is created by :meth:`Connection.blobopen` and provides
   access to a blob in the database.  It behaves like a Python file.
   At the C level it wraps a `sqlite3_blob
-  <http://sqlite.org/c3ref/blob.html>`_.
+  <https://sqlite.org/c3ref/blob.html>`_.
 
   .. note::
 
     You cannot change the size of a blob using this object. You should
     create it with the correct size in advance either by using
     :class:`zeroblob` or the `zeroblob()
-    <http://www.sqlite.org/lang_corefunc.html>`_ function.
+    <https://sqlite.org/lang_corefunc.html>`_ function.
 
   See the :ref:`example <example-blobio>`.
 */
@@ -210,7 +210,7 @@ APSWBlob_close_internal(APSWBlob *self, int force)
   PyObject *err_type, *err_value, *err_traceback;
 
   if(force==2)
-    PyErr_Fetch(&err_type, &err_value, &err_traceback);    
+    PyErr_Fetch(&err_type, &err_value, &err_traceback);
 
   /* note that sqlite3_blob_close always works even if an error is
      returned - see sqlite ticket #2815 */
@@ -231,8 +231,8 @@ APSWBlob_close_internal(APSWBlob *self, int force)
               break;
             case 2:
               SET_EXC(res, self->connection->db);
-              apsw_write_unraiseable(NULL);            
-            }         
+              apsw_write_unraiseable(NULL);
+            }
         }
       self->pBlob=0;
     }
@@ -242,7 +242,7 @@ APSWBlob_close_internal(APSWBlob *self, int force)
      we'd still be in list */
   if(self->connection)
     Connection_remove_dependent(self->connection, (PyObject*)self);
-  
+
   Py_CLEAR(self->connection);
 
   if(force==2)
@@ -264,15 +264,13 @@ APSWBlob_dealloc(APSWBlob *self)
 
 /* If the blob is closed, we return the same error as normal python files */
 #define CHECK_BLOB_CLOSED \
-  if(!self->pBlob) \
-    { \
-      PyErr_Format(PyExc_ValueError, "I/O operation on closed blob"); \
-      return NULL; \
-    }
+  do { if(!self->pBlob)							\
+    return PyErr_Format(PyExc_ValueError, "I/O operation on closed blob"); \
+  } while(0)
 
 /** .. method:: length() -> int
 
-  Returns the size of the blob in bytes. 
+  Returns the size of the blob in bytes.
 
   -* sqlite3_blob_bytes
 */
@@ -307,7 +305,7 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
 
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
-   
+
   /* The python file read routine treats negative numbers as read till
      end of file, which I think is rather silly.  (Try reading -3
      bytes from /dev/zero on a 64 bit machine with lots of swap to see
@@ -336,6 +334,8 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
 
   thebuffer= PyBytes_AS_STRING(buffy);
   PYSQLITE_BLOB_CALL(res=sqlite3_blob_read(self->pBlob, thebuffer, length, self->curoffset));
+  if(PyErr_Occurred())
+    return NULL;
 
   if(res!=SQLITE_OK)
     {
@@ -349,6 +349,102 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
   return buffy;
 }
 
+/** .. method:: readinto(buffer[, offset=0, length=remaining-buffer]) -> None
+
+  Reads from the blob into a buffer you have supplied.  This method is
+  useful if you already have a buffer like object that data is being
+  assembled in, and avoids allocating results in :meth:`blob.read` and
+  then copying into buffer.
+
+  :param buffer: A writable buffer like object.  In Python 2.6 onwards
+                 there is a bytearray type that is very useful.
+                 :class:`array.array` also works.
+
+  :param offset: The position to start writing into the buffer
+                 defaulting to the beginning.
+
+  :param length: How much of the blob to read.  The default is the
+                 remaining space left in the buffer.  Note that if
+                 there is more space available than blob left then you
+                 will get a :exc:`ValueError` exception.
+
+  -* sqlite3_blob_read
+*/
+
+static PyObject*
+APSWBlob_readinto(APSWBlob *self, PyObject *args)
+{
+  int length;
+  int res;
+  Py_ssize_t offset, lengthwanted;
+  PyObject *wbuf=NULL;
+
+  int aswb;
+  void *buffer;
+  Py_ssize_t bufsize;
+
+  int bloblen;
+
+  CHECK_USE(NULL);
+  CHECK_BLOB_CLOSED;
+
+  /* To get Py_ssize_t we need "n" format but that only exists in
+     Python 2.5 plus */
+
+  if(!PyArg_ParseTuple(args, "O|"
+#if PY_VERSION_HEX < 0x02050000
+		       "i"
+#else
+		       "n"
+#endif
+		       "i:readinto(wbuf, offset=1, length=wbufremaining)",
+		       &wbuf, &offset, &length))
+    return NULL;
+
+  aswb=PyObject_AsWriteBuffer(wbuf, &buffer, &bufsize);
+  if(aswb)
+    return NULL;
+
+  /* Although a lot of these checks could be combined into a single
+     one, they are kept separate so that we can verify they have each
+     been exercised with code coverage checks */
+
+  if(PyTuple_GET_SIZE(args)<2)
+    offset=0;
+
+  bloblen=sqlite3_blob_bytes(self->pBlob);
+
+  if(offset<0 || offset>bufsize)
+    return PyErr_Format(PyExc_ValueError, "offset is less than zero or beyond end of buffer");
+
+  if(PyTuple_GET_SIZE(args)<3)
+    lengthwanted=bufsize-offset;
+  else
+    lengthwanted=length;
+
+  if(lengthwanted<0)
+    return PyErr_Format(PyExc_ValueError, "Length wanted is negative");
+
+  if(offset+lengthwanted>bufsize)
+    return PyErr_Format(PyExc_ValueError, "Data would go beyond end of buffer");
+
+  if(lengthwanted>bloblen-self->curoffset)
+    return PyErr_Format(PyExc_ValueError, "More data requested than blob length");
+
+  PYSQLITE_BLOB_CALL(res=sqlite3_blob_read(self->pBlob, (char*)buffer+offset, lengthwanted, self->curoffset));
+  if(PyErr_Occurred())
+    return NULL;
+
+  if(res!=SQLITE_OK)
+    {
+      SET_EXC(res, self->connection->db);
+      return NULL;
+    }
+  self->curoffset+=lengthwanted;
+
+  Py_RETURN_NONE;
+}
+
 /** .. method:: seek(offset[, whence=0]) -> None
 
   Changes current position to *offset* biased by *whence*.
@@ -366,15 +462,14 @@ APSWBlob_seek(APSWBlob *self, PyObject *args)
   int offset, whence=0;
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
-  
+
   if(!PyArg_ParseTuple(args, "i|i:seek(offset,whence=0)", &offset, &whence))
     return NULL;
-  
+
   switch(whence)
     {
     default:
-      PyErr_Format(PyExc_ValueError, "whence parameter should be 0, 1 or 2");
-      return NULL;
+      return PyErr_Format(PyExc_ValueError, "whence parameter should be 0, 1 or 2");
     case 0: /* relative to begining of file */
       if(offset<0 || offset>sqlite3_blob_bytes(self->pBlob))
         goto out_of_range;
@@ -393,8 +488,7 @@ APSWBlob_seek(APSWBlob *self, PyObject *args)
     }
   Py_RETURN_NONE;
  out_of_range:
-  PyErr_Format(PyExc_ValueError, "The resulting offset would be less than zero or past the end of the blob");
-  return NULL;
+  return PyErr_Format(PyExc_ValueError, "The resulting offset would be less than zero or past the end of the blob");
 }
 
 /** .. method:: tell() -> int
@@ -445,23 +539,17 @@ APSWBlob_write(APSWBlob *self, PyObject *obj)
         return NULL;
     }
   else
-    {
-      PyErr_Format(PyExc_TypeError, "Parameter should be bytes/string or buffer");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_TypeError, "Parameter should be bytes/string or buffer");
 
   if( ((int)(size+self->curoffset))<self->curoffset)
-    {
-      PyErr_Format(PyExc_ValueError, "Data is too large (integer wrap)");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_ValueError, "Data is too large (integer wrap)");
+
   if( ((int)(size+self->curoffset))>sqlite3_blob_bytes(self->pBlob))
-    {
-      PyErr_Format(PyExc_ValueError, "Data would go beyond end of blob");
-      return NULL;
-    }
+    return PyErr_Format(PyExc_ValueError, "Data would go beyond end of blob");
 
   PYSQLITE_BLOB_CALL(res=sqlite3_blob_write(self->pBlob, buffer, size, self->curoffset));
+  if(PyErr_Occurred())
+    return NULL;
 
   if(res!=SQLITE_OK)
     {
@@ -477,7 +565,7 @@ APSWBlob_write(APSWBlob *self, PyObject *obj)
 /** .. method:: close([force=False])
 
   Closes the blob.  Note that even if an error occurs the blob is
-  still closed (see :cvstrac:`2815`).
+  still closed.
 
   .. note::
 
@@ -515,7 +603,7 @@ APSWBlob_close(APSWBlob *self, PyObject *args)
   Py_RETURN_NONE;
 }
 
-/** .. method:: __enter__() -> context 
+/** .. method:: __enter__() -> context
 
   You can use a blob as a `context manager
   <http://docs.python.org/reference/datamodel.html#with-statement-context-managers>`_
@@ -562,17 +650,67 @@ APSWBlob_exit(APSWBlob *self, APSW_ARGUNUSED PyObject *args)
   Py_RETURN_FALSE;
 }
 
+/** .. method:: reopen(rowid)
+
+  Change this blob object to point to a different row.  It can be
+  faster than closing an existing blob an opening a new one.
+
+  -* sqlite3_blob_reopen
+*/
+
+static PyObject *
+APSWBlob_reopen(APSWBlob *self, PyObject *arg)
+{
+  int res;
+  long long rowid;
+
+  CHECK_USE(NULL);
+  CHECK_BLOB_CLOSED;
+
+#if PY_MAJOR_VERSION<3
+  if(PyInt_Check(arg))
+    rowid=PyInt_AS_LONG(arg);
+  else
+#endif
+    if (PyLong_Check(arg))
+      {
+	rowid=PyLong_AsLongLong(arg);
+	if(PyErr_Occurred())
+	  return NULL;
+      }
+    else
+      return PyErr_Format(PyExc_TypeError, "blob reopen argument must be a number");
+
+  /* no matter what happens we always reset current offset */
+  self->curoffset=0;
+
+  PYSQLITE_BLOB_CALL(res=sqlite3_blob_reopen(self->pBlob, rowid));
+  if(PyErr_Occurred())
+    return NULL;
+
+  if(res!=SQLITE_OK)
+    {
+      SET_EXC(res, self->connection->db);
+      return NULL;
+    }
+  Py_RETURN_NONE;
+}
+
 static PyMethodDef APSWBlob_methods[]={
   {"length", (PyCFunction)APSWBlob_length, METH_NOARGS,
    "Returns length in bytes of the blob"},
   {"read", (PyCFunction)APSWBlob_read, METH_VARARGS,
    "Reads data from the blob"},
+  {"readinto", (PyCFunction)APSWBlob_readinto, METH_VARARGS,
+   "Reads data from the blob into a provided buffer"},
   {"seek", (PyCFunction)APSWBlob_seek, METH_VARARGS,
    "Seeks to a position in the blob"},
   {"tell", (PyCFunction)APSWBlob_tell, METH_NOARGS,
    "Returns current blob offset"},
   {"write", (PyCFunction)APSWBlob_write, METH_O,
    "Writes data to blob"},
+  {"reopen", (PyCFunction)APSWBlob_reopen, METH_O,
+   "Changes the blob to point to a different row"},
   {"close", (PyCFunction)APSWBlob_close, METH_VARARGS,
    "Closes blob"},
   {"__enter__", (PyCFunction)APSWBlob_enter, METH_NOARGS,
@@ -587,7 +725,7 @@ static PyTypeObject APSWBlobType = {
     "apsw.blob",               /*tp_name*/
     sizeof(APSWBlob),          /*tp_basicsize*/
     0,                         /*tp_itemsize*/
-    (destructor)APSWBlob_dealloc, /*tp_dealloc*/ 
+    (destructor)APSWBlob_dealloc, /*tp_dealloc*/
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
@@ -631,4 +769,3 @@ static PyTypeObject APSWBlobType = {
     0                          /* tp_del */
     APSW_PYTYPE_VERSION
 };
-
